#include "../C-Engine/CEngine/Object.hpp"
#include "../C-Engine/CEngine/Engine.hpp"
#include "../C-Engine/CEngine/Script.hpp"

#include "../C-Engine/CPhysics/Transforms.hpp"
#include "../C-Engine/CPhysics/Rigibodys.hpp"

#include "../C-Engine/CRendering/Render.hpp"
#include "../C-Engine/CRendering/Animation.hpp"
#include "../C-Engine/CRendering/Sprite.hpp"
#include "../C-Engine/CRendering/TilesMapping.hpp"

extern "C"
{
#include <fxlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../C-Engine/CExtraLib/input.h"
#include "../C-Engine/CExtraLib/MonochromeLib.h"
#include "../C-Engine/CExtraLib/FonctionC.h"
}


Object::Object()
{
    OTransforms = new Transforms(this);
    ORender = new Render(this);
    ORigibody = NULL;
    OScript = NULL;
    Conteneur = NULL;
    enable = true;
    tag = new char[10];
}

Object::~Object()
{
if(OTransforms != NULL) OTransforms->~Transforms();
if(ORender != NULL)     ORender->~Render();
if(ORigibody != NULL)   ORigibody->~Rigibody();
if(OScript != NULL)     OScript->~Script();
if(tag != NULL)         free(tag);
}

void Object::AffectScript(Script & VScript)
{
    OScript = &VScript;
    OScript->AffectObject(this);
}

void Object::AffectScript(Script * VScript)
{
    OScript = VScript;
    VScript->AffectObject(this);
}

 void Object::AffectEngine(Engine * EV)
 {
     Conteneur = EV;
 }

void Object::AffectTag(char * v)
 {
    strcpy(tag , v);
 }

void Object::EnableObject()
{
    this->enable = true;
}

void Object::DisableObject()
{
    this->enable = false;
}

bool Object::GetEnable()
{
    return this->enable;
}

bool Object::IsOnScreen()
{
    if(OTransforms->GetX() < Conteneur->screen.max.x && OTransforms->GetX() + ORender->GetSizeX() > Conteneur->screen.min.x && OTransforms->GetY() < Conteneur->screen.max.y && OTransforms->GetY() + ORender->GetSizeY()  > Conteneur->screen.min.y ) return true;
    else return false;
}

Object *  Object::CollisionTagO(char* v)
{

    for(int i = 0; i< Conteneur->GetNbObject(); i++)
    {
        if(strcmp(Conteneur->ListeObject[i]->tag , v) == 0 && Conteneur->ListeObject[i]->enable == true)
        {
           if( ((   Conteneur->ListeObject[i]->OTransforms->GetX() + Conteneur->ListeObject[i]->ORender->GetBlitX() >= OTransforms->GetX() + ORender->GetSizeX() + ORender->GetBlitX())
                 || (Conteneur->ListeObject[i]->OTransforms->GetX() + Conteneur->ListeObject[i]->ORender->GetSizeX() + Conteneur->ListeObject[i]->ORender->GetBlitX()<= OTransforms->GetX() + ORender->GetBlitX())
                 || (Conteneur->ListeObject[i]->OTransforms->GetY() + Conteneur->ListeObject[i]->ORender->GetBlitY()>= OTransforms->GetY() + ORender->GetSizeY() + ORender->GetBlitY())
                 || (Conteneur->ListeObject[i]->OTransforms->GetY() + Conteneur->ListeObject[i]->ORender->GetSizeY() + Conteneur->ListeObject[i]->ORender->GetBlitY()<= OTransforms->GetY() + ORender->GetBlitY())
                ) == false )
            {
            return Conteneur->ListeObject[i];
            }
        }
    }

return NULL;
}


bool Object::CollisionTag(char* v)
{

    for(int i = 0; i< Conteneur->GetNbObject(); i++)
    {
        if(strcmp(Conteneur->ListeObject[i]->tag , v) == 0 && Conteneur->ListeObject[i]->enable == true)
        {
           if( ((   Conteneur->ListeObject[i]->OTransforms->GetX() + Conteneur->ListeObject[i]->ORender->GetBlitX() >= OTransforms->GetX() + ORender->GetSizeX() + ORender->GetBlitX())
                 || (Conteneur->ListeObject[i]->OTransforms->GetX() + Conteneur->ListeObject[i]->ORender->GetSizeX() + Conteneur->ListeObject[i]->ORender->GetBlitX()<= OTransforms->GetX() + ORender->GetBlitX())
                 || (Conteneur->ListeObject[i]->OTransforms->GetY() + Conteneur->ListeObject[i]->ORender->GetBlitY()>= OTransforms->GetY() + ORender->GetSizeY() + ORender->GetBlitY())
                 || (Conteneur->ListeObject[i]->OTransforms->GetY() + Conteneur->ListeObject[i]->ORender->GetSizeY() + Conteneur->ListeObject[i]->ORender->GetBlitY()<= OTransforms->GetY() + ORender->GetBlitY())
                ) == false )
            {
            return true;
            }
        }
    }

return false;
}

bool Object::PixelPerfect(char * v)
{

for(int i = 0; i< Conteneur->GetNbObject(); i++)
    {
        if(strcmp(Conteneur->ListeObject[i]->tag , v) == 0 && Conteneur->ListeObject[i]->enable == true)
        {
           if( ((   Conteneur->ListeObject[i]->OTransforms->GetX() + Conteneur->ListeObject[i]->ORender->GetBlitX() >= OTransforms->GetX() + ORender->GetSizeX() + ORender->GetBlitX())
                 || (Conteneur->ListeObject[i]->OTransforms->GetX() + Conteneur->ListeObject[i]->ORender->GetSizeX() + Conteneur->ListeObject[i]->ORender->GetBlitX()<= OTransforms->GetX() + ORender->GetBlitX())
                 || (Conteneur->ListeObject[i]->OTransforms->GetY() + Conteneur->ListeObject[i]->ORender->GetBlitY()>= OTransforms->GetY() + ORender->GetSizeY() + ORender->GetBlitY())
                 || (Conteneur->ListeObject[i]->OTransforms->GetY() + Conteneur->ListeObject[i]->ORender->GetSizeY() + Conteneur->ListeObject[i]->ORender->GetBlitY()<= OTransforms->GetY() + ORender->GetBlitY())
                ) == false )
            {
                //Test pixel Perfect
                //Utiliser le canal Alpha et la lib Collider.





            }
        }
    }

return false;


}

